5G-AKA Tamarin file README
==========================================

This is the README for the Tamarin files associated with the NDSS submission 
''Component-Based Formal Analysis of 5G-AKA: Channel Assumptions and Session Confusion''.
These models are of the 5G-AKA protocol, under various threat models.

**TL;DR: Install Tamarin. Run `make` on this directory. Visit [http://127.0.0.1:3001/](http://127.0.0.1:3001/)**


Tamarin Installation
--------------------

To run these files, you will need the Tamarin Prover tool installed.

Please follow the [instruction instructions within the Tamarin Manual
(link)](https://tamarin-prover.github.io/manual/book/002_installation.html).

- On Mac, (with Homebrew installed) this is as simple as:

>     brew update >     brew install tamarin-prover/tap/tamarin-prover


- On Linux, you can either use LinuxBrew, or download the the binary from   the
following location:   https://github.com/tamarin-prover/tamarin-prover/releases

- Or, with LinuxBrew installed, run:

>     brew update >     brew install tamarin-prover/tap/tamarin-prover

For both, if you want to run the latest version (not necessary, but slightly
faster), please build Tamarin from source (see the manual).



Running Tamarin to verify these files
-------------------------------------

If you have both m4 and make installed, please then run

>     make

on this directory. This will generate the correct output files.

Once this is running, please then visit
[http://127.0.0.1:3001/](http://127.0.0.1:3001/) in your browser.

This will open the interactive Tamarin web GUI, where you will be able to
interact with the lemmas and generate proofs.


You may have to click ''Allow the application tamarin-prover to accept incoming
network connections''. If port 3001 is already being used for something on your
system, please change this port number in the Makefile.

If you do not have both `m4` and `make` installed, please copy and paste the
following command into a terminal in this directory:

>     tamarin-prover interactive . --port=3001 --heuristic=i

and then point your browser to [http://localhost:3001/](http://localhost:3001/).
This will run the Tamarin Prover with the correct heuristic against the current
folder (and the `*.spthy` files in it), and then open the interactive Tamarin
web-GUI.

Within this web-page, please then click `5G_aka`.

These are the main models, and have been automatically generated from the `m4`
file.

By default, without changing any of the flags in the `5G_aka.m4` file, this will
present the naive models of 5G-AKA, precisely as defined by TS 33.501 v0.7.0.
This (`5G_aka.m4`) is the main file you want to read, to inspect the models.

Notes on the presented models and lemmas.
---

Please feel free to go through the lemmas in browser, and autoprove them.

You will note there are broadly two versions of each lemma: `lemma_name`,
followed by `lemma_name_restricted`. Where a lemma is proven true, it must not
be done in the restricted setting. The normal (unrestricted) lemmas work without
any restrictions on the number of runs of a protocol, or number of participants
able to take part. These are necessary for proving a property about the protocol
true.

Where the lemma is not true, we use a restricted lemma which only allows a
finite (often very small) number of actors to take part in the protocol run.
This restricted version of the protocol is then very often able to find the
counter-example to the lemma much more quickly and efficiently than in the
unrestricted case. Lemmas falsified in the restricted case are equally false in
the unrestricted case, and vice-versa, lemmas proven true in the UN-restricted
case are necessarily true in the restricted case.

The existence of a `_restricted` lemma does not indicate that the lemma is
untrue: we include restricted versions for all lemmas. However there are threat
models (e.g. compromise of all ARPFs) where most of the lemmas are easily
falsified.

Many lemmas also have a comment next to them in the original `m4` (or resultant
`.spthy`) file. This will often indicate which heuristic is best to prove the
lemma under. We have set the default heuristic (in the `Makefile`) to `s`, as
this works well for many lemmas. Frustratingly, one of the helper lemmas,
`sqn_ue_increase` does not terminate under heuristic `s`; this lemma is true,
and terminates almost instantly under heuristic `S`. Please feel free to verify
this. We remind readers that choice of heuristic does not affect the validity of
the result, only whether Tamarin terminates or not.

If a lemma does not terminate, please check the source models to see if there is
a comment associated with it, indicating a preferred heuristic. If there is
none, please at a minimum attempt to switch between `s`, `S`, and `i` (and as a
last resort, `c`) to see if that improves matters. Except where results indicate
otherwise, all lemmas terminate giving results in at least one heuristic. (i.e.
if the lemma is true, the non-restricted lemma terminates; if the lemma is
false, one of the two presented lemmas gives the attack trace.)

The main `5G_aka` file is provided as an m4 file, as we use m4 macros to enable
and disable various options within the models, and to rename some of the
protocol rules and ordering for the sake of proof-speed.
We also provide `5G-aka.spthy` pre-generated (for users without `m4`), but it is
not recommended that you attempt to read this version. Feel free to run 
`m4 5G_aka.m4 > 5g_aka.spthy` to confirm that the latter file was actually
generated by the former.

Please note that upon loading various files and models into Tamarin (especially
ones with channel or component compromise), various helper and sources lemmas
are required for the models, and to aid termination. Tamarin pre-computes
sources before opening the models in interactive mode, which may cause Tamarin
to take a few seconds to load; output of this source computation will be visible
on the terminal. This is completely normal.


Channel and Component Compromise
---

To enable channel or component compromise, please un-comment the relevant `dnl`
commented lines to be found within the `5G_aka.m4` file. This will cause a new
model to be generated, with a different threat model / adversarial capabilities,
as described in Section X of the paper. Please follow the instructions in this
section precisely, as inadvertently enabling multiple of these different threat
models will most likely break the overall models, preventing them from working
as intended. Please do not attempt to enable both channel and component
compromise at the same time.

Within channel and/or component compromise, the reader can enable compromise of
different channels as described in the paper, e.g. Full DY compromise or read-
only compromise of one or both channels, and compromise of different core
components. The reader can also enable All-X for component compromise, or Not-
My-X for component compromise, again, as described in the paper. Please only
enable one channel or component compromise macro at a time.

To return to the non-compromised models, please enable (i.e. un-comment)
`define(normal_channels, ``'')`, and then choose one of `core_naive_channels`
or `core_session_bound_channels`, with or without the `SNID_fix`.

The `core_naive_channels` are precisely as defined by TS 33.501 v0.7.0, and the
models generated under `core_session_bound_channels` are those for our proposed
session-binding fix.

Please do not delete or move the `channel_compromise.m4i` and
`component_compromise.m4i` files, as these are necessary for the channel and
component compromise models and lemmas.

Manual proofs (for verification of the lemma or discovery of counterexample) are
needed for many of the lemmas under the component compromise threat models. A
brief guide to proof strategies (i.e. which goals to solve in which order in
Tamarin interactively in browser) for many lemmas is provided in 
`manual-solving-notes.md`.
